"""
Japan Electricity Utilities Web APIs

List of Utilities:
    - Kyushu Electric
    - Chubu Electric
    - Tokyo Electric
"""

import pandas as pd
from io import StringIO
import requests
import datetime

#region Kyuden
def __call_api_kyushuElectric(endpoint, column_names=None, skiprows=7,nrows=24):
    url = f"http://www.kyuden.co.jp/power_usages/csv/juyo-hourly-{endpoint}.csv"
    s = StringIO(requests.get(url).content.decode("Shift JIS")) #Use Japanese encoding
    try:
        if column_names==None:
            return pd.read_csv(s, skiprows=skiprows,nrows=nrows)
        else:
            return pd.read_csv(s, header=None,skiprows=skiprows+1, nrows=nrows,names=column_names)
    except Exception as e:
        print(e)
        return None

def kyushuElectricDemandJapanese(day="20190101"):
    return __call_api_kyushuElectric(endpoint=f"{day}")

def kyushuElectricDemandEnglish(day="20190101"):
    return __call_api_kyushuElectric(endpoint=f"{day}",column_names=["date", "time", "actual demand 10,000 kW", "forecast 10,000 kW","Usage (%)"])

def kyushuElectricDemand5MSJapanese(day="20190101"):
    return __call_api_kyushuElectric(endpoint=f"{day}",skiprows=42,nrows=288)

def kyushuElectricDemand5MSEnglish(day="20190101"):
    return __call_api_kyushuElectric(endpoint=f"{day}",skiprows=42,nrows=288,column_names=["date","time","5-Minute Actuals 10,000 kW"])

#endregion Kyuden

#region Chuden
def __call_api_chubuElectric(column_names=None, skiprows=0):
    url = f"http://denki-yoho.chuden.jp/denki_yoho_content_data/areajuyo_current.csv"
    s = StringIO(requests.get(url).content.decode("Shift JIS")) #Use Japanese encoding
    try:
        if column_names==None:
            return pd.read_csv(s, skiprows=skiprows)
        else:
            return pd.read_csv(s, header=None,skiprows=skiprows+1,names=column_names)
    except Exception as e:
        print(e)
        return None

def chubuElectricDemandJapanese():
    return __call_api_chubuElectric()

def chubuElectricDemandEnglish():
    return __call_api_chubuElectric(column_names=['DATE','Time',"Actual Demand 10,000 kW"])

def chubuElectricDemandJapaneseRange(begtime = "20180101",endtime="20180201"):
    df = __call_api_chubuElectric()
    df.set_index('DATE', inplace=True)
    begtime = datetime.datetime.strptime(begtime, '%Y%m%d')
    endtime = datetime.datetime.strptime(endtime, '%Y%m%d')
    return df.loc[begtime.strftime('%Y/%m/%d'):endtime.strftime('%Y/%m/%d')]

def chubuElectricDemandEnglishRange(begtime = "20180101",endtime="20180201"):
    df = __call_api_chubuElectric(column_names=['DATE','Time',"Actual Demand 10,000 kW"])
    df.set_index('DATE', inplace=True)
    begtime = datetime.datetime.strptime(begtime, '%Y%m%d')
    endtime = datetime.datetime.strptime(endtime, '%Y%m%d')
    return df.loc[begtime.strftime('%Y/%m/%d'):endtime.strftime('%Y/%m/%d')]

#endregion Chuden

#region TEPCO
def __call_api_TEPCO(endpoint, column_names=None, skiprows=7,nrows=24):
    url =f"http://www.tepco.co.jp/forecast/html/images/juyo-{endpoint}.csv"
    s = StringIO(requests.get(url).content.decode("Shift JIS"))  # Use Japanese encoding
    try:
        if column_names == None:
            return pd.read_csv(s, skiprows=skiprows, nrows=nrows)
        else:
            return pd.read_csv(s, header=None, skiprows=skiprows + 1, nrows=nrows, names=column_names)
    except Exception as e:
        print(e)
        return None

def tepcoElectricDemandHistoricalJapanese(year="2019"):
    return __call_api_TEPCO(endpoint=f"{year}",skiprows=2,nrows=9000)

def tepcoElectricDemandHistoricalEnglish(year="2019"):
    return __call_api_TEPCO(endpoint=f"{year}",skiprows=2,nrows=9000,column_names=["date", "time", "actual demand 10,000 kW"])

def tepcoElectricDemandCurrentJapanese():
    return __call_api_TEPCO(endpoint=f"j")

def tepcoElectricDemandCurrentEnglish():
    return __call_api_TEPCO(endpoint=f"j",column_names=["date", "time", "actual demand 10,000 kW","forecast 10,000 kW","Usage (%)"])

#endregion TEPCO




"""
Australian NG Market General APIs

List of APIs:
    - Gas Scheme Register
"""


import pandas as pd

def __call_api_AEMC():
    url = "https://www.aemc.gov.au/energy-system/gas/gas-scheme-register"
    df = pd.read_html(url, index_col="Title")
    df[0].columns = ['State', 'Type', 'Operator', 'Comments']
    return df[0]

def pipelineRegister():
    return __call_api_AEMC()

def pipelineRegisterSearch(state="",operator="",type=""):
    df = __call_api_AEMC()

    if state!="":
        df = df.loc[df["State"].str.contains(state)]

    if operator!="":
        df = df.loc[df["Operator"].str.contains(operator)]

    if type!="":
        df = df.loc[df["Type"].str.contains(type)]

    return df





import pandas as pd
from datetime import timedelta
from datetime import datetime
import numpy as np

def __call_api(endpoint,date_str):
    url = f"https://www.emcsg.com/marketdata/{endpoint}"

    try:
        df = pd.read_html(url)
        numberTables = df.__len__()
        lastTable = numberTables - 1
        df1 = df[lastTable]
        #convert date row to datetime then set it to the correct index
        for i, row in df1.iterrows():
            minutes_add = ((df1.at[i, 'PERIOD'] * 30) - 30)
            df1.at[i, 'DATE'] = datetime.strptime(date_str, '%Y-%m-%d') + timedelta(
                minutes=minutes_add.astype(np.float64))
        df1.set_index('DATE', inplace=True)
        return df1
    except Exception as e:
        print(e)
        return None


def singaporeUSEP(date="2019-01-01"):
    datetime_obj = datetime.strptime(date, '%Y-%m-%d')
    date_str = datetime.strftime(datetime_obj, '%d+%b+%Y')

    endpoint = f"priceinformation?doAccessData=true&USEP_accessAction=dataView&USEP_date={date_str}&dataType=USEP"
    return __call_api(endpoint,date)




"""
U.S. Energy Information Administration
This is an API wrapper  for the eia-python API wrapper.

Contains select calls to distinct Data Series, and returns pandas dataFrames

"""
import eia
import pandas as pd
from .common import commonData


def __call_api(report,key="",series=None):
    api = eia.API(key)
    if series==None:
        series_search = api.data_by_series(series=commonData.dictEiaReports.get(report))
    else:
        series_search = api.data_by_series(series=series)

    result = pd.DataFrame(series_search)
    return result

def eiaAPI(seriesID="",key=""):
    return __call_api(report = None,key =key,series=seriesID)

def coalAustraliaProduction(key=""):
    return __call_api(report="australiaCoalProduction",key=key,series=None)

def coalAustraliaConsumption(key=""):
    return __call_api(report="australiaCoalConsumption",key=key,series=None)

def ngAustraliaProduction(key=""):
    return __call_api(report="australiaNGProduction",key=key,series=None)

def ngHenryHubSpotDaily(key=""):
    return __call_api(report="henryHubNGSpotPriceDaily",key=key,series=None)

def crudeCushingSpotDaily(key=""):
    return __call_api(report="wtiCushingSpotPriceDaily",key=key,series=None)


